@@ -27,6 +27,7 @@ import gnomeapplet
 import gobject
 import gtk
 from gtk import gdk
+import pango
 
 import os
 from xml.sax import saxutils
@@ -89,11 +90,28 @@ class MusicApplet:
         if not USE_GTK_TOOLTIP:
             self.__tooltips = gtk.Tooltips ()
 
+        if USE_GTK_TOOLTIP:
+            self.__applet.props.has_tooltip = True
+            self.__applet.connect ("query-tooltip", self.__query_tooltip_cb)
+        else:
+            song_tip = SongTooltip (self.__song_tip_guts)
+            song_tip.set_tip (self.__applet, "")
+
+        # Song scroller
+
+        self.__scroller = musicapplet.widgets.Scroller ()
+        self.__make_transparent (self.__scroller)
+        full_key = self.__conf.resolve_key ("show_song_information")
+        self.__conf.client.notify_add (full_key,
+                lambda client, id, entry, unused:
+                    entry.value is not None and self.__update_scroller_visibility (entry.value.get_bool ()))
+
         # Rating
 
         self.__rating = Rating ()
 	self.__set_widget_tooltip (self.__rating, _("Rate current song"))
         self.__rating.connect ("notify::rating", self.__rate_song_cb)
+        self.__make_transparent (self.__rating)
         full_key = self.__conf.resolve_key ("show_rating")
         self.__conf.client.notify_add (full_key,
                 lambda client, id, entry, unused:
@@ -102,15 +120,7 @@ class MusicApplet:
         # Time display
 
         self.__time = gtk.Label ("--:--")
-        self.__time.show ()
-        self.__event_box = gtk.EventBox ()
-        if USE_GTK_TOOLTIP:
-            self.__event_box.props.has_tooltip = True
-            self.__event_box.connect ("query-tooltip", self.__query_tooltip_cb)
-        else:
-            song_tip = SongTooltip (self.__song_tip_guts)
-            song_tip.set_tip (self.__event_box, "")
-        self.__event_box.add (self.__time)
+        self.__make_transparent (self.__time)
         full_key = self.__conf.resolve_key ("show_time")
         self.__conf.client.notify_add (full_key,
                 lambda client, id, entry, unused:
@@ -155,8 +165,6 @@ class MusicApplet:
         # Putting it all together
 
         self.__change_orient_cb (applet, applet.get_orient ())
-        self.__event_box.set_style (applet.get_style ())
-        self.__rating.set_style (applet.get_style ())
 
         applet.connect ("change-background", self.__change_background_cb)
         applet.connect ("change-orient", self.__change_orient_cb)
@@ -167,6 +175,9 @@ class MusicApplet:
         self.__manager.connect ("notify::preferred",
                                 lambda manager, pspec: self.__set_preferred (manager.preferred))
 
+    def __make_transparent (self, widget):
+        widget.connect ("realize", lambda widget: widget.window.set_back_pixmap (None, True))
+
     def __set_widget_tooltip (self, widget, text):
         if USE_GTK_TOOLTIP:
 	    widget.set_tooltip_text (text)
@@ -203,9 +214,11 @@ class MusicApplet:
             self.__plugin_handlers = []
             self.__rating.interactive = False
 
+        self.__scroller.props.plugin = plugin
         self.__song_tip_guts.set_plugin (plugin)
         if self.__notify is not None:
             self.__notify.set_plugin (plugin)
+        self.__update_scroller_visibility ()
         self.__update_rating_visibility ()
         self.__update_time_visibility ()
         self.__update_controls_visibility ()
@@ -228,6 +241,14 @@ class MusicApplet:
 	    self.__set_widget_tooltip (self.__launch, _("Launch preferred player"))
 
 
+    def __update_scroller_visibility (self, conf_value=None):
+        full_key = self.__conf.resolve_key ("show_song_information")
+        if conf_value is None:
+            conf_value = self.__conf.client.get_bool (full_key)
+        conf_value = self.__assure_not_invisible (full_key, conf_value)
+        self.__scroller.props.visible = self.__plugin is not None and conf_value
+
+
     def __update_rating_visibility (self, conf_value=None):
         full_key = self.__conf.resolve_key ("show_rating")
         if conf_value is None:
@@ -241,7 +262,7 @@ class MusicApplet:
         if conf_value is None:
             conf_value = self.__conf.client.get_bool (full_key)
         conf_value = self.__assure_not_invisible (full_key, conf_value)
-        self.__event_box.set_property ("visible", self.__plugin is not None and conf_value)
+        self.__time.set_property ("visible", self.__plugin is not None and conf_value)
 
 
     def __update_controls_visibility (self, conf_value=None):
@@ -255,7 +276,7 @@ class MusicApplet:
 
     def __assure_not_invisible (self, latest_full_key, conf_value):
         visible = False
-        for key in ["show_rating", "show_time", "show_controls"]:
+        for key in ["show_song_information", "show_rating", "show_time", "show_controls"]:
             full_key = self.__conf.resolve_key (key)
             visible = visible or self.__conf.client.get_bool (full_key)
         if not visible:
@@ -270,21 +291,24 @@ class MusicApplet:
 
 
     def __change_background_cb (self, applet, type, color, pixmap):
-        for widget in [applet, self.__event_box, self.__rating]:
-            widget.set_style (None)
-            widget.modify_style (gtk.RcStyle ())
-            if type == gnomeapplet.NO_BACKGROUND:
-                pass
-            elif type == gnomeapplet.COLOR_BACKGROUND:
-                widget.modify_bg (gtk.STATE_NORMAL, color)
-            elif type == gnomeapplet.PIXMAP_BACKGROUND:
-                style = widget.get_style ().copy ()
-                style.bg_pixmap[gtk.STATE_NORMAL] = pixmap
-                widget.set_style (style)
+        applet.set_style (None)
+        applet.modify_style (gtk.RcStyle ())
+
+        if type == gnomeapplet.NO_BACKGROUND:
+            pass
+        elif type == gnomeapplet.COLOR_BACKGROUND:
+            applet.modify_bg (gtk.STATE_NORMAL, color)
+        elif type == gnomeapplet.PIXMAP_BACKGROUND:
+            style = applet.get_style ().copy ()
+            style.bg_pixmap[gtk.STATE_NORMAL] = pixmap
+            applet.set_style (style)
+
+        for widget in [self.__scroller, self.__time, self.__rating]:
+            widget.queue_draw ()
 
 
     def __change_orient_cb (self, applet, orient):
-        widgets = [self.__rating, self.__event_box, self.__previous, self.__play, self.__next, self.__launch]
+        widgets = [self.__scroller, self.__rating, self.__time, self.__previous, self.__play, self.__next, self.__launch]
         if self.__all is not None:
             for widget in widgets:
                 self.__all.remove (widget)
@@ -294,9 +318,11 @@ class MusicApplet:
             self.__all = gtk.HBox ()
         else:
             self.__all = gtk.VBox ()
+        self.__make_transparent (self.__all)
 
+        self.__all.pack_start (self.__scroller, padding=3)
         self.__all.pack_start (self.__rating)
-        self.__all.pack_start (self.__event_box, padding=3)
+        self.__all.pack_start (self.__time, padding=3)
         self.__all.pack_start (self.__previous)
         self.__all.pack_start (self.__play)
         self.__all.pack_start (self.__next)
@@ -308,6 +334,7 @@ class MusicApplet:
             self.__time.set_angle (270)
         else:
             self.__time.set_angle (0)
+        self.__scroller.props.orientation = orient
 
         self.__rating.vertical = (orient == gnomeapplet.ORIENT_LEFT or orient == gnomeapplet.ORIENT_RIGHT)
         for button in [self.__previous, self.__play, self.__next, self.__launch]:
@@ -428,7 +455,7 @@ class MusicApplet:
             self.__about_dialog = gtk.AboutDialog ()
             self.__about_dialog.set_name (_("Music Applet"))
             self.__about_dialog.set_version (musicapplet.defs.VERSION)
-            self.__about_dialog.set_copyright (_("(C) 2007 Paul Kuliniewicz"))
+            self.__about_dialog.set_copyright (_("(C) 2008 Paul Kuliniewicz"))
             self.__about_dialog.set_comments (_("Control your favorite music player from a GNOME panel."))
             self.__about_dialog.set_website ("http://www.kuliniewicz.org/music-applet/")
             self.__about_dialog.set_logo (self.__about_dialog.render_icon (STOCK_ICON, gtk.ICON_SIZE_LARGE_TOOLBAR))
@@ -617,12 +644,13 @@ class Rating (gtk.EventBox):
         gtk.EventBox.__init__ (self)
         self.add_events (gdk.POINTER_MOTION_MASK)
 
-        color = self.get_style ().text[gtk.STATE_NORMAL]
-        self.__set_star   = self.create_colorized_pixbuf (STOCK_SET_STAR, color)
-        self.__unset_star = self.create_colorized_pixbuf (STOCK_UNSET_STAR, color)
-
+        self.__set_star_stock = self.render_icon (STOCK_SET_STAR, gtk.ICON_SIZE_MENU)
+        self.__unset_star_stock = self.render_icon (STOCK_UNSET_STAR, gtk.ICON_SIZE_MENU)
+        self.__set_star = self.__set_star_stock
+        self.__unset_star = self.__unset_star_stock
         self.__star_width = max (self.__set_star.get_width (), self.__unset_star.get_width ())
         self.__star_height = max (self.__set_star.get_height (), self.__unset_star.get_height ())
+
         self.__rating = -1.0
         self.__minimum = -1.0
         self.__maximum = -1.0
@@ -631,6 +659,17 @@ class Rating (gtk.EventBox):
         self.__interactive = False
 
 
+    def do_style_set (self, old_style):
+        gtk.EventBox.do_style_set (self, old_style)
+
+        # render_icon causes this to be called, so...
+        if "_Rating__set_star_stock" in dir (self) and "_Rating__unset_star_stock" in dir (self):
+            color = self.get_style ().fg[gtk.STATE_NORMAL]
+            self.__set_star   = self.create_colorized_pixbuf (self.__set_star_stock, color)
+            self.__unset_star = self.create_colorized_pixbuf (self.__unset_star_stock, color)
+            self.queue_draw ()
+
+
     def do_get_property (self, property):
         if property.name == "rating":
             return self.__rating
@@ -759,19 +798,22 @@ class Rating (gtk.EventBox):
             self.queue_draw ()
 
 
-    def create_colorized_pixbuf (self, stock_id, color):
-        pixbuf = self.render_icon (stock_id, gtk.ICON_SIZE_MENU)
+    def create_colorized_pixbuf (self, stock_pixbuf, color):
+        pixbuf = stock_pixbuf.copy ()
+        red_scale = color.red / 65535.0
+        green_scale = color.green / 65535.0
+        blue_scale = color.blue / 65535.0
         for row in pixbuf.get_pixels_array ():
             for pixel in row:
                 # Yay API changes!
                 if str (type (pixel[0])) == "<type 'array'>":
-                    pixel[0][0] *= color.red
-                    pixel[1][0] *= color.green
-                    pixel[2][0] *= color.blue
+                    pixel[0][0] *= red_scale
+                    pixel[1][0] *= green_scale
+                    pixel[2][0] *= blue_scale
                 else:
-                    pixel[0] *= color.red
-                    pixel[1] *= color.green
-                    pixel[2] *= color.blue
+                    pixel[0] *= red_scale
+                    pixel[1] *= green_scale
+                    pixel[2] *= blue_scale
         return pixbuf
 
 
@@ -943,6 +985,7 @@ def scale_pixbuf (pixbuf):
 
 
 
+gobject.type_register (NotifyPopup)
 gobject.type_register (Rating)
 register_stock_icons ()
 augment_icon_path ()
