@@ -41,7 +41,7 @@ class Manager (gtk.ListStore):
     """
 
     COL_PATH = 0
-    COL_NAME = 1
+    COL_INTERNAL_NAME = 1
     COL_ICON_NAME = 2
     COL_MODULE = 3
     COL_OBJECT = 4
@@ -49,6 +49,7 @@ class Manager (gtk.ListStore):
     COL_HANDLER = 6
     COL_ERROR = 7
     COL_DIALOG = 8
+    COL_DISPLAY_NAME = 9
 
 
     __gproperties__ = {
@@ -68,15 +69,16 @@ class Manager (gtk.ListStore):
 
 
     def __init__ (self, conf, dirs):
-        gtk.ListStore.__init__ (self, str,
-                                      str,
-                                      str,
-                                      gobject.TYPE_PYOBJECT,
-                                      gobject.TYPE_PYOBJECT,
-                                      bool,
-                                      int,
-                                      str,
-                                      gobject.TYPE_PYOBJECT)
+        gtk.ListStore.__init__ (self, str,                      # COL_PATH
+                                      str,                      # COL_INTERNAL_NAME
+                                      str,                      # COL_ICON_NAME
+                                      gobject.TYPE_PYOBJECT,    # COL_MODULE
+                                      gobject.TYPE_PYOBJECT,    # COL_OBJECT
+                                      bool,                     # COL_ENABLED
+                                      int,                      # COL_HANDLER
+                                      str,                      # COL_ERROR
+                                      gobject.TYPE_PYOBJECT,    # COL_DIALOG
+                                      str)                      # COL_DISPLAY_NAME
         self.__active = None
         self.__preferred = None
         self.__conf = conf
@@ -90,7 +92,7 @@ class Manager (gtk.ListStore):
                         path = os.path.join (dir, file)
                         module = pydoc.importfile (path)
                         module.INFO["path"] = path
-                        name = module.INFO["name"]
+                        name = module.INFO.get ("internal-name") or module.INFO["name"]
                         if name not in avail.keys () or self.__better_plugin (module, avail[name]):
                             avail[name] = module
                     except Exception, detail:
@@ -104,7 +106,16 @@ class Manager (gtk.ListStore):
         names.sort (key=str.lower)
         for name in names:
             module = avail[name]
-            self.append ((module.INFO["path"], name, module.INFO["icon-name"], module, None, False, 0, "", None))
+            self.append ((module.INFO["path"],
+                          module.INFO.get ("internal-name") or module.INFO["name"],
+                          module.INFO["icon-name"],
+                          module,
+                          None,
+                          False,
+                          0,
+                          "",
+                          None,
+                          module.INFO.get ("display-name") or module.INFO["name"]))
 
         full_key = conf.resolve_key ("enabled_plugins")
         self.foreach (self.__set_enabled, (conf.client.get_list (full_key, gconf.VALUE_STRING), False))
@@ -161,8 +172,8 @@ class Manager (gtk.ListStore):
         remain unenabled.
         """
 
-        (name, module, object, enabled) = self.get (
-                iter, self.COL_NAME, self.COL_MODULE, self.COL_OBJECT, self.COL_ENABLED)
+        (internal_name, module, object, enabled) = self.get (
+                iter, self.COL_INTERNAL_NAME, self.COL_MODULE, self.COL_OBJECT, self.COL_ENABLED)
         if enabled:
             return
 
@@ -170,8 +181,10 @@ class Manager (gtk.ListStore):
             if object is None:
                 object = module.create_instance (self.__conf)
                 handler = object.connect ("notify::connected", self.__plugin_connected_cb)
-                self.set (iter, self.COL_OBJECT, object, self.COL_HANDLER, handler,
-                                self.COL_ICON_NAME, module.INFO["icon-name"], self.COL_ERROR, "")
+                self.set (iter, self.COL_OBJECT, object,
+                                self.COL_HANDLER, handler,
+                                self.COL_ICON_NAME, module.INFO["icon-name"],
+                                self.COL_ERROR, "")
         
             self.set_value (iter, self.COL_ENABLED, True)
             if self.__active is None:
@@ -181,8 +194,8 @@ class Manager (gtk.ListStore):
 
             full_key = self.__conf.resolve_key ("enabled_plugins")
             enabled_names = self.__conf.client.get_list (full_key, gconf.VALUE_STRING)
-            if name not in enabled_names:
-                enabled_names.append (name)
+            if internal_name not in enabled_names:
+                enabled_names.append (internal_name)
                 self.__conf.client.set_list (full_key, gconf.VALUE_STRING, enabled_names)
 
         except Exception, error:
@@ -195,7 +208,7 @@ class Manager (gtk.ListStore):
         Disable a plugin.
         """
 
-        (name, object, enabled) = self.get (iter, self.COL_NAME, self.COL_OBJECT, self.COL_ENABLED)
+        (internal_name, object, enabled) = self.get (iter, self.COL_INTERNAL_NAME, self.COL_OBJECT, self.COL_ENABLED)
         if not enabled:
             return
 
@@ -212,8 +225,8 @@ class Manager (gtk.ListStore):
 
         full_key = self.__conf.resolve_key ("enabled_plugins")
         enabled_names = self.__conf.client.get_list (full_key, gconf.VALUE_STRING)
-        if name in enabled_names:
-            enabled_names.remove (name)
+        if internal_name in enabled_names:
+            enabled_names.remove (internal_name)
             self.__conf.client.set_list (full_key, gconf.VALUE_STRING, enabled_names)
 
 
@@ -224,8 +237,8 @@ class Manager (gtk.ListStore):
         """
 
         (enabled_names, activate) = data
-        name = model.get_value (iter, model.COL_NAME)
-        if name in enabled_names:
+        internal_name = model.get_value (iter, model.COL_INTERNAL_NAME)
+        if internal_name in enabled_names:
             self.__enable_plugin (iter, activate)
         else:
             self.__disable_plugin (iter)
@@ -308,13 +321,13 @@ class Manager (gtk.ListStore):
             return False
 
 
-    def __update_preferred (self, name):
+    def __update_preferred (self, internal_name):
         """
         Update the preferred property based on the name of the preferred
         player.
         """
 
-        iter = musicapplet.util.search_model (self, self.COL_NAME, name)
+        iter = musicapplet.util.search_model (self, self.COL_INTERNAL_NAME, internal_name)
         if iter is not None:
             self.preferred = self.get_value (iter, self.COL_OBJECT)
 
@@ -338,7 +351,7 @@ def create_plugins_dialog (manager, conf):
             (module, object, enabled, error) = model.get (
                     iter, model.COL_MODULE, model.COL_OBJECT, model.COL_ENABLED, model.COL_ERROR)
             xml.get_widget ("name").set_markup ("<big><b>" +
-                                                _("%s Plugin") % saxutils.escape (module.INFO["name"]) +
+                                                _("%s Plugin") % saxutils.escape (module.INFO.get ("display-name") or module.INFO["name"]) +
                                                 "</b></big>")
             for key in keys:
                 xml.get_widget (key).set_text (module.INFO[key])
@@ -394,7 +407,7 @@ def create_plugins_dialog (manager, conf):
     plugins.append_column (column)
 
     renderer = gtk.CellRendererText ()
-    column = gtk.TreeViewColumn ("Name", renderer, text=manager.COL_NAME)
+    column = gtk.TreeViewColumn ("Name", renderer, text=manager.COL_DISPLAY_NAME)
     plugins.append_column (column)
 
     selection = plugins.get_selection ()
@@ -414,10 +427,10 @@ def create_plugins_dialog (manager, conf):
 
     renderer = gtk.CellRendererText ()
     preferred.pack_start (renderer, True)
-    preferred.set_attributes (renderer, text=manager.COL_NAME)
+    preferred.set_attributes (renderer, text=manager.COL_DISPLAY_NAME)
 
     full_key = conf.resolve_key ("preferred_player")
-    conf.bind_combo_string (full_key, preferred, manager.COL_NAME)
+    conf.bind_combo_string (full_key, preferred, manager.COL_INTERNAL_NAME)
 
     # Everything else
 
