@@ -56,10 +56,16 @@ class PluginBase (gobject.GObject):
     __gproperties__ = {
         "player_name" : (gobject.TYPE_STRING,
                          "player_name",
-                         "The name of the player that this plugin connects to",
+                         "The (display) name of the player that this plugin connects to",
                          None,
                          gobject.PARAM_READABLE),
 
+        "internal_name" : (gobject.TYPE_STRING,
+                           "internal_name",
+                           "The internal name of the plugin",
+                           None,
+                           gobject.PARAM_READABLE),
+
         "icon_name" : (gobject.TYPE_STRING,
                        "icon_name",
                        "The name of the icon that represents the player",
@@ -134,6 +140,7 @@ class PluginBase (gobject.GObject):
 
 
     player_name = musicapplet.util.make_property ("player_name")
+    internal_name = musicapplet.util.make_property ("internal_name")
     icon_name = musicapplet.util.make_property ("icon_name")
     connected = musicapplet.util.make_property ("connected")
     playing = musicapplet.util.make_property ("playing")
@@ -155,11 +162,20 @@ class PluginBase (gobject.GObject):
     ##################################################################
 
 
-    def __init__ (self, conf, player_name, icon_name=None):
+    def __init__ (self, conf, names, icon_name=None):
         gobject.GObject.__init__ (self)
         self._conf = conf
+
+        if str (type (names)) == "<type 'dict'>":
+            player_name = names["display"]
+            internal_name = names["internal"]
+        else:
+            player_name = names
+            internal_name = names
+
         self.__values = {
                 "player-name" : player_name,
+                "internal-name" : internal_name,
                 "icon-name" : icon_name,
                 "connected" : False,
                 "playing" : False,
@@ -285,8 +301,8 @@ class DBusPlugin (PluginBase):
     """
 
 
-    def __init__ (self, conf, player_name, icon_name, dbus_name):
-        PluginBase.__init__ (self, conf, player_name, icon_name)
+    def __init__ (self, conf, names, icon_name, dbus_name):
+        PluginBase.__init__ (self, conf, names, icon_name)
 
         import dbus
         import dbus.glib
@@ -430,8 +446,8 @@ class MPRISPlugin (DBusPlugin):
     STATUS_STOPPED = 2
 
 
-    def __init__ (self, conf, player_name, icon_name, dbus_name):
-        DBusPlugin.__init__ (self, conf, player_name, icon_name, dbus_name)
+    def __init__ (self, conf, names, icon_name, dbus_name):
+        DBusPlugin.__init__ (self, conf, names, icon_name, dbus_name)
 
         self.__dbus_name = dbus_name
         self.__player = None
